SystemOrganization addCategory: #'Tweak-Core-Proto'!

TextAttribute subclass: #CFieldDefinitionTextAttribute
	instanceVariableNames: 'fieldClass fieldDefinition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Core-Proto'!

!CFieldDefinitionTextAttribute class methodsFor: 'instance creation' stamp: 'ar 5/5/2004 17:27'!
forField: aField in: aClass
	^(self new)
		fieldClass: aClass;
		fieldDefinition: aField;
		yourself! !

!CFieldDefinitionTextAttribute methodsFor: 'emphasis' stamp: 'ar 5/6/2004 15:32'!
emphasizeScanner: scanner
	"Set the emphasis for text display"
	fieldDefinition isGeneratingEvents
		ifTrue:[scanner addEmphasis: 1]. "bold"
	fieldDefinition isVirtualField
		ifTrue:[scanner addEmphasis: 2]. "italic"
	fieldDefinition isInstanceVariable
		ifTrue:[scanner textColor: Color red]
		ifFalse:[(fieldDefinition toGet == nil and:[fieldDefinition toSet == nil]) ifTrue:[
			scanner addEmphasis: 16. "struck out"
		]].
	fieldDefinition color ifNotNil:[
		scanner textColor: (Color black mixed: 0.5 with: fieldDefinition color).
	].! !

!CFieldDefinitionTextAttribute methodsFor: 'accessing' stamp: 'ar 5/5/2004 17:27'!
fieldClass
	^fieldClass! !

!CFieldDefinitionTextAttribute methodsFor: 'accessing' stamp: 'ar 5/5/2004 17:27'!
fieldClass: aClass
	fieldClass := aClass.! !

!CFieldDefinitionTextAttribute methodsFor: 'accessing' stamp: 'ar 5/5/2004 17:27'!
fieldDefinition
	^fieldDefinition! !

!CFieldDefinitionTextAttribute methodsFor: 'accessing' stamp: 'ar 5/5/2004 17:27'!
fieldDefinition: aFieldDef
	fieldDefinition := aFieldDef! !

!CFieldDefinitionTextAttribute methodsFor: 'accessing' stamp: 'ar 5/5/2004 21:05'!
menu
	^fieldClass fieldMenuFor: fieldDefinition! !

!CFieldDefinitionTextAttribute methodsFor: 'storing' stamp: 'ar 4/28/2005 12:16'!
writeScanOn: strm
	"this text attribute is ignored"! !

Exception subclass: #CSqueak3Point8Detected
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Core-Proto'!

!CSqueak3Point8Detected commentStamp: '<historical>' prior: 0!
An exception used to figure out and workaround some problems when we are running under Squeak 3.8.!

!CSqueak3Point8Detected methodsFor: 'as yet unclassified' stamp: 'ar 1/29/2005 15:39'!
defaultAction
	self resume.! !

XMLWriter subclass: #XMLWriterIndenting
	instanceVariableNames: 'indent tight lineEnd'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Core-Proto'!

!XMLWriterIndenting methodsFor: 'writing xml' stamp: 'ar 2/28/2004 20:41'!
endTag
	indent := indent + 1.
	super endTag.! !

!XMLWriterIndenting methodsFor: 'writing xml' stamp: 'bf 4/25/2005 13:11'!
endTag: tagName
	tight
		ifTrue: [tight := false]
		ifFalse: [stream crtab: indent-1].
	indent := indent - 2.
	super endTag: tagName.! !

!XMLWriterIndenting methodsFor: 'initialize' stamp: 'ar 3/15/2007 18:22'!
initialize
	super initialize.
	indent := 0.
	tight := false.
	lineEnd := String cr.! !

!XMLWriterIndenting methodsFor: 'accessing' stamp: 'ar 3/15/2007 18:21'!
lineEnd
	^lineEnd! !

!XMLWriterIndenting methodsFor: 'accessing' stamp: 'ar 3/15/2007 18:22'!
lineEnd: aString
	lineEnd := aString! !

!XMLWriterIndenting methodsFor: 'writing xml' stamp: 'bf 4/25/2005 14:42'!
pcData: aString
	super pcData: aString.
	tight := true.
! !

!XMLWriterIndenting methodsFor: 'writing xml' stamp: 'bf 4/25/2005 14:44'!
putAsXMLString: aValue
	self stream nextPut: $".
	super pcData: aValue.	"need to factor out data escaping in superclass"
	self stream nextPut: $"! !

!XMLWriterIndenting methodsFor: 'writing xml' stamp: 'ar 3/15/2007 18:22'!
startTag: tagName
	tight ifFalse: [stream nextPutAll: lineEnd; tab: indent].
	super startTag: tagName.! !

Object subclass: #CFieldDefinition
	instanceVariableNames: 'home name flags type toGet toSet accessKey changeEvent serializer deserializer color'
	classVariableNames: 'FlagInstanceVariable FlagNonSerializable FlagPrivate FlagVirtual'
	poolDictionaries: ''
	category: 'Tweak-Core-Proto'!

!CFieldDefinition commentStamp: 'tk 7/27/2004 16:03' prior: 0!
CFieldDefinition contains the definition of fields. WARNING: This class is fundamental and I still need to do some cleanup and reformulation. You can look but you can't touch ;-) at least not before discussing the changes.


name		of the virtual variable stored in properties
flags		bits for (FlagVirtual, FlagInstanceVariable, FlagNonSerializable, FlagPrivate)
type		Class we expect the value to be
toGet		selector used to get the value
toSet		selector used to set the value
accessKey		
changeEvent		
serializer		
deserializer		
color		How it shows in the class definition, in the browser.

whyText	Dictionary of (key -> text of explanation).  Allows different levels of explanation for this variable.
!

!CFieldDefinition class methodsFor: 'islands' stamp: 'ar 5/22/2005 15:32'!
howToPassAsArgument
	^#passByIdentity:! !

!CFieldDefinition class methodsFor: 'class initialization' stamp: 'ar 5/5/2004 20:42'!
initialize
	"CFieldDefinition initialize"

	"FlagInstanceVariable:
		This flag determines whether the field is an instance variable."
	FlagInstanceVariable := 1.

	"FlagVirtual:
		This flag marks fields delegating to some other object."
	FlagVirtual := 2.

	"FlagPrivate:
		This flag marks private fields."
	FlagPrivate := 4.

	"FlagNonSerializable:
		This flag marks non-serializable fields."
	FlagNonSerializable := 8.
! !

!CFieldDefinition class methodsFor: 'instance creation' stamp: ''!
named: aString
	^self new name: aString asSymbol! !

!CFieldDefinition class methodsFor: 'instance creation' stamp: 'ar 1/30/2005 11:49'!
new
	^self basicNew initialize! !

!CFieldDefinition methodsFor: 'comparing' stamp: ''!
= aFieldDef
	^aFieldDef class == self class and:[self name = aFieldDef name and:[self flags = aFieldDef flags]]! !

!CFieldDefinition methodsFor: 'accessing' stamp: ''!
accessKey
	"Answer the access key that is used to index the field some object"
	^accessKey! !

!CFieldDefinition methodsFor: 'accessing' stamp: ''!
accessKey: anObject
	"Set the access key that is used to index the field some object"
	accessKey := anObject! !

!CFieldDefinition methodsFor: 'private' stamp: 'ar 3/26/2004 15:13'!
assignDefaultSetter
	"Private. Assign a default setter for the field. This is needed by the compiler for being able to compile 'forward references' of fields."
	toSet := (self name,':') asSymbol.
! !

!CFieldDefinition methodsFor: 'initialize' stamp: 'ar 5/5/2004 21:54'!
beInstanceVariable
	"Make me into an instance variable"
	self isInstanceVariable: true.
	self isVirtualField: false.
	self serializable: true.
	self changeEvent: nil.
	self accessKey: nil.
	self toGet: name.
	self toSet: (name,':') asSymbol.
! !

!CFieldDefinition methodsFor: 'initialize' stamp: 'ar 5/5/2004 21:53'!
beRegularField
	"Make me into a virtual field"
	self isInstanceVariable: false.
	self isVirtualField: false.
	self serializable: true.
	self changeEvent: (name, 'Changed') asSymbol.
	self accessKey: nil.
	self toGet: name.
	self toSet: (name,':') asSymbol.
! !

!CFieldDefinition methodsFor: 'initialize' stamp: 'ar 5/5/2004 21:54'!
beVirtualField
	"Make me into a virtual field"
	self isInstanceVariable: false.
	self isVirtualField: true.
	self serializable: false.
	self changeEvent: nil.
	self accessKey: nil.
	self toGet: name.
	self toSet: (name,':') asSymbol.
! !

!CFieldDefinition methodsFor: 'accessing' stamp: 'ar 3/9/2003 22:57'!
changeEvent
	^changeEvent ifNil:[(self name, 'Changed') asSymbol]! !

!CFieldDefinition methodsFor: 'accessing' stamp: 'ar 3/9/2003 22:57'!
changeEvent: aSymbol
	changeEvent := aSymbol.! !

!CFieldDefinition methodsFor: 'accessing' stamp: 'ar 5/6/2004 12:31'!
color
	^color! !

!CFieldDefinition methodsFor: 'accessing' stamp: 'ar 5/6/2004 18:07'!
color: aColor
	color := aColor ifNotNil:[aColor asColor].! !

!CFieldDefinition methodsFor: 'accessing' stamp: 'ar 9/22/2003 00:35'!
deserializer
	^deserializer! !

!CFieldDefinition methodsFor: 'accessing' stamp: 'ar 9/22/2003 00:35'!
deserializer: aSymbol
	deserializer := aSymbol! !

!CFieldDefinition methodsFor: 'islands' stamp: 'ar 6/18/2005 14:20'!
exportOnIslandWriter: writer
	^writer exportFieldDefinition: self! !

!CFieldDefinition methodsFor: 'private' stamp: 'ar 1/1/2003 20:55'!
flags
	^flags! !

!CFieldDefinition methodsFor: 'private' stamp: 'ar 1/1/2003 20:55'!
flags: newFlags
	flags := newFlags.! !

!CFieldDefinition methodsFor: 'comparing' stamp: ''!
hash
	^name hash! !

!CFieldDefinition methodsFor: 'accessing' stamp: 'ar 5/26/2005 00:27'!
home
	^home! !

!CFieldDefinition methodsFor: 'accessing' stamp: 'ar 5/26/2005 00:27'!
home: aClass
	home := aClass! !

!CFieldDefinition methodsFor: 'private' stamp: 'ar 5/6/2004 12:35'!
importXmlFrom: xmlNode
	"Read an XML representation of the receiver which can be used to fool old tools"
	| val |
	xmlNode name == #field 
		ifFalse:[^self error: 'Cannot read field from #', xmlNode name].
	name := xmlNode contentString asSymbol.
	(val := xmlNode attributeAt: 'instanceVariable')
		ifNotNil:[self isInstanceVariable: (xmlNode literalAt: 'instanceVariable')].
	(val := xmlNode attributeAt: 'virtual')
		ifNotNil:[self isVirtualField: (xmlNode literalAt: 'virtual')].
	(val := xmlNode attributeAt: 'private')
		ifNotNil:[self isPrivateField: (xmlNode literalAt: 'private')].
	(val := xmlNode attributeAt: 'serializable')
		ifNotNil:[self serializable: (xmlNode literalAt: 'serializable')].
	(val := xmlNode attributeAt: 'type')
		ifNotNil:[type := Smalltalk at: val asSymbol ifAbsent:[nil]].
	(val := xmlNode attributeAt: 'toGet')
		ifNotNil:[toGet := val asSymbol].
	(val := xmlNode attributeAt: 'toSet')
		ifNotNil:[toSet := val asSymbol].
	(val := xmlNode attributeAt: 'accessKey')
		ifNotNil:[accessKey := xmlNode literalAt: 'accessKey'].
	(val := xmlNode attributeAt: 'changeEvent')
		ifNotNil:[changeEvent := val asSymbol].
	(val := xmlNode attributeAt: 'serializer')
		ifNotNil:[serializer := val asSymbol].
	(val := xmlNode attributeAt: 'deserializer')
		ifNotNil:[deserializer := val asSymbol].
	(val := xmlNode attributeAt: 'color')
		ifNotNil:[color := Color fromHtmlColorRef: val].! !

!CFieldDefinition methodsFor: 'initialize' stamp: ''!
initialize
	"Initialize the receiver"
	flags := 0.! !

!CFieldDefinition methodsFor: 'testing' stamp: 'ar 5/5/2004 20:53'!
isGeneratingEvents
	^self isRegularField and:[changeEvent notNil]! !

!CFieldDefinition methodsFor: 'accessing' stamp: ''!
isInstanceVariable
	^(flags bitAnd: FlagInstanceVariable) == FlagInstanceVariable! !

!CFieldDefinition methodsFor: 'accessing' stamp: ''!
isInstanceVariable: aBool
	flags := aBool 	ifTrue:[flags bitOr: FlagInstanceVariable] 
					ifFalse:[flags bitClear: FlagInstanceVariable]! !

!CFieldDefinition methodsFor: 'accessing' stamp: ''!
isPrivateField
	"Answer whether this field is private"
	^(flags bitAnd: FlagPrivate) == FlagPrivate! !

!CFieldDefinition methodsFor: 'accessing' stamp: ''!
isPrivateField: aBool
	"Answer whether this field is private"
	flags := aBool ifTrue:[flags bitOr: FlagPrivate] ifFalse:[flags bitClear: FlagPrivate].! !

!CFieldDefinition methodsFor: 'testing' stamp: 'ar 3/10/2003 01:54'!
isReadOnly
	^toSet == nil! !

!CFieldDefinition methodsFor: 'testing' stamp: 'ar 5/5/2004 20:54'!
isRegularField
	^(self isVirtualField or:[self isInstanceVariable]) not! !

!CFieldDefinition methodsFor: 'testing' stamp: 'ar 5/5/2004 20:51'!
isSerializable
	"Answer whether this field should be serialized"
	^self isVirtualField not and:[self serializable]! !

!CFieldDefinition methodsFor: 'testing' stamp: 'ar 5/5/2004 18:41'!
isTyped
	^type notNil! !

!CFieldDefinition methodsFor: 'accessing' stamp: 'ar 5/5/2004 20:47'!
isVirtualField
	"Answer whether this field is private"
	^(flags bitAnd: FlagVirtual) == FlagVirtual! !

!CFieldDefinition methodsFor: 'accessing' stamp: 'ar 5/5/2004 20:48'!
isVirtualField: aBool
	"Answer whether this field is virtual"
	flags := aBool ifTrue:[flags bitOr: FlagVirtual] ifFalse:[flags bitClear: FlagVirtual].! !

!CFieldDefinition methodsFor: 'accessing' stamp: 'ar 1/1/2003 20:09'!
key
	"for compatibility with bindings"
	^self name asSymbol! !

!CFieldDefinition methodsFor: 'accessing' stamp: ''!
name
	^name! !

!CFieldDefinition methodsFor: 'accessing' stamp: ''!
name: aString
	name := aString! !

!CFieldDefinition methodsFor: 'accessing' stamp: 'ar 5/5/2004 20:49'!
nonSerializable
	^(flags bitAnd: FlagNonSerializable) == FlagNonSerializable! !

!CFieldDefinition methodsFor: 'accessing' stamp: 'ar 5/5/2004 20:50'!
nonSerializable: aBool
	"Answer whether this field is non-serializable"
	flags := aBool ifTrue:[flags bitOr: FlagNonSerializable] ifFalse:[flags bitClear: FlagNonSerializable].! !

!CFieldDefinition methodsFor: 'printing' stamp: 'ar 5/29/2005 18:20'!
printOn: aStream
	aStream nextPutAll: '['; 
		nextPutAll: home name;
		nextPutAll: '''s ';
		nextPutAll: name asString; 
		nextPutAll:']'.! !

!CFieldDefinition methodsFor: 'private' stamp: 'ar 5/5/2004 22:15'!
readXmlFrom: aStream
	"Read an XML representation of the receiver which can be used to fool old tools"
	| doc node |
	doc := XMLDOMParser parseDocumentFrom: aStream.
	node := doc elements first.
	self importXmlFrom: node.! !

!CFieldDefinition methodsFor: 'accessing' stamp: 'ar 5/5/2004 20:50'!
serializable
	^self nonSerializable not! !

!CFieldDefinition methodsFor: 'accessing' stamp: 'ar 5/5/2004 20:50'!
serializable: aBool
	self nonSerializable: aBool not! !

!CFieldDefinition methodsFor: 'accessing' stamp: 'ar 9/22/2003 00:35'!
serializer
	^serializer! !

!CFieldDefinition methodsFor: 'accessing' stamp: 'ar 9/22/2003 00:35'!
serializer: aSymbol
	serializer := aSymbol! !

!CFieldDefinition methodsFor: 'private' stamp: 'ar 5/6/2004 12:35'!
storeXmlOn: writer
	"Answer an XML representation of the receiver which can be used to fool old tools"
	writer startTag: 'field'.
	self isInstanceVariable
		ifTrue:[writer attribute: 'instanceVariable' value: 'true'].
	self isVirtualField
		ifTrue:[writer attribute: 'virtual' value: 'true'].
	self isPrivateField
		ifTrue:[writer attribute: 'private' value: 'true'].
	self nonSerializable
		ifTrue:[writer attribute: 'serializable' value: 'false'].
	self isTyped
		ifTrue:[writer attribute: 'type' value: type name].
	toGet isNil
		ifFalse:[writer attribute: 'toGet' value: toGet].
	toSet isNil
		ifFalse:[writer attribute: 'toSet' value: toSet].
	accessKey isNil
		ifFalse:[writer attribute: 'accessKey' value: accessKey printString].
	changeEvent isNil
		ifFalse:[writer attribute: 'changeEvent' value: changeEvent].
	serializer isNil
		ifFalse:[writer attribute: 'serializer' value: serializer].
	deserializer isNil
		ifFalse:[writer attribute: 'deserializer' value: deserializer].
	color isNil
		ifFalse:[writer attribute: 'color' value: color asHtmlColorRef].
	writer endTag.
	writer pcData: name.
	writer endTag: 'field'.! !

!CFieldDefinition methodsFor: 'accessing' stamp: ''!
toGet
	"Answer the selector used to retrieve the value of this field"
	^toGet! !

!CFieldDefinition methodsFor: 'accessing' stamp: ''!
toGet: aSymbol
	"Set the selector used to retrieve the value of this field"
	toGet := aSymbol! !

!CFieldDefinition methodsFor: 'accessing' stamp: ''!
toSet
	"Answer the selector used to store the value of this field"
	^toSet! !

!CFieldDefinition methodsFor: 'accessing' stamp: ''!
toSet: aSymbol
	"Set the selector used to store the value of this field"
	toSet := aSymbol.! !

!CFieldDefinition methodsFor: 'accessing' stamp: 'ar 5/5/2004 18:41'!
type
	^type! !

!CFieldDefinition methodsFor: 'accessing' stamp: 'ar 5/5/2004 18:41'!
type: aType
	type := aType.! !

!CFieldDefinition methodsFor: 'accessing' stamp: 'ar 5/5/2004 20:48'!
valueType
	"Answer the type of the object that may be stored in the field described by the receiver"
	^type! !

!CFieldDefinition methodsFor: 'private' stamp: 'ar 5/5/2004 22:06'!
xmlString
	"Answer an XML representation of the receiver which can be used to fool old tools"
	^String streamContents:[:s| self storeXmlOn: s].! !

Object subclass: #CMethodTrigger
	instanceVariableNames: 'home selector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Core-Proto'!

!CMethodTrigger commentStamp: '<historical>' prior: 0!
CMethodTrigger is the superclass for all method triggers. Method triggers are specifications to send specific messages (resulting in the evaluation of the corresponding methods, hence the name) when "certain things happen", such as an event being triggered.

Instance variables:
	where		<Class>		The class where the method is defined
	selector		<Symbol>	The selector of the method being triggered.!

CMethodTrigger subclass: #CEventTrigger
	instanceVariableNames: 'eventName field'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Core-Proto'!

!CEventTrigger commentStamp: 'ar 4/28/2004 02:30' prior: 0!
CEventTrigger is a method trigger which responds to particular events. The event might be generated in the corresponding object or in one of its fields. Event triggers are typically specified using the annotations <on: eventName> or <on: eventName in: fieldName>.

Instance variables:
	eventName	<Symbol>	The name of the event triggering the script.
	fieldName	<Symbol>	The name of the field generating the event.
	changeEvent <Symbol>	The name of the change event associated with the field.
!

!CEventTrigger methodsFor: 'comparing' stamp: 'ar 2/28/2005 15:21'!
= aTrigger
	^(super = aTrigger) 
		and:[eventName == aTrigger eventName
		and:[field = aTrigger field]]! !

!CEventTrigger methodsFor: 'accessing' stamp: 'ar 2/28/2005 15:21'!
changeEvent
	^field ifNotNil:[field changeEvent]! !

!CEventTrigger methodsFor: 'accessing' stamp: 'ar 4/27/2004 20:40'!
eventName
	^eventName! !

!CEventTrigger methodsFor: 'accessing' stamp: 'ar 4/27/2004 20:46'!
eventName: aString
	eventName := aString asSymbol.! !

!CEventTrigger methodsFor: 'accessing' stamp: 'ar 2/28/2005 15:21'!
field
	^field! !

!CEventTrigger methodsFor: 'accessing' stamp: 'ar 2/28/2005 15:21'!
field: aFieldDefinition
	field := aFieldDefinition.! !

!CEventTrigger methodsFor: 'accessing' stamp: 'ar 2/28/2005 15:21'!
fieldName
	^field ifNotNil:[field name]! !

!CEventTrigger methodsFor: 'comparing' stamp: 'ar 4/27/2004 21:09'!
hash
	^super hash bitXor: eventName identityHash! !

!CEventTrigger methodsFor: 'starting' stamp: 'ar 5/7/2005 18:26'!
installIn: receiver
	"Install the trigger in the given receiver"
	| eventSource script |
	field ifNil:[eventSource := receiver] ifNotNil:[eventSource := nil].
	script := receiver startScript: selector when: {eventSource. eventName}.
	script myTrigger: self.
	^script! !

!CEventTrigger methodsFor: 'printing' stamp: 'ar 5/29/2005 18:29'!
printOn: aStream
	aStream nextPutAll: self class name.
	aStream nextPut: $(.
	aStream nextPutAll: 'on: '.
	aStream nextPutAll: eventName.
	field ifNotNil:[
		aStream nextPutAll: ' in: '.
		aStream nextPutAll: self fieldName.
	].
	aStream nextPutAll: ' -> '.
	aStream nextPutAll: home name.
	aStream nextPutAll: '>>'.
	aStream print: selector.
	aStream nextPut: $).
! !

!CMethodTrigger class methodsFor: 'islands' stamp: 'ar 5/22/2005 15:32'!
howToPassAsArgument
	^#passByIdentity:! !

!CMethodTrigger methodsFor: 'comparing' stamp: 'ar 4/27/2004 20:43'!
= aTrigger
	"Compare method triggers"
	self class == aTrigger class ifFalse:[^false].
	^selector == aTrigger selector! !

!CMethodTrigger methodsFor: 'accessing' stamp: 'ar 4/24/2005 04:10'!
eventName
	^self subclassResponsibility! !

!CMethodTrigger methodsFor: 'islands' stamp: 'ar 6/18/2005 14:21'!
exportOnIslandWriter: writer
	^writer exportMethodTrigger: self! !

!CMethodTrigger methodsFor: 'comparing' stamp: 'ar 4/27/2004 20:42'!
hash
	"hash is implemented because = is implemented"
	^selector identityHash! !

!CMethodTrigger methodsFor: 'accessing' stamp: 'ar 5/26/2005 00:26'!
home
	^home! !

!CMethodTrigger methodsFor: 'accessing' stamp: 'ar 5/26/2005 00:26'!
home: aClass
	home := aClass! !

!CMethodTrigger methodsFor: 'starting' stamp: 'ar 7/28/2004 22:56'!
installIn: receiver
	"Install this trigger for the given receiver"
	^self subclassResponsibility! !

!CMethodTrigger methodsFor: 'accessing' stamp: 'ar 4/27/2004 20:43'!
selector
	^selector! !

!CMethodTrigger methodsFor: 'accessing' stamp: 'ar 4/27/2004 20:52'!
selector: aString
	selector := aString asSymbol.! !

!CMethodTrigger methodsFor: 'starting' stamp: 'tk 5/19/2005 14:31'!
startUpIn: receiver
	"Install and start this trigger for the given receiver"
	| aScript |
	(aScript := self installIn: receiver) autoStartTrigger.
	^ aScript! !

CMethodTrigger subclass: #CTickingTrigger
	instanceVariableNames: 'frequency'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Core-Proto'!

!CTickingTrigger commentStamp: 'ar 4/28/2004 02:31' prior: 0!
CTickingTrigger is the specification for a "ticking" script, e.g., a script which is evaluated repeatedly with a certain frequency. Ticking scripts are typically specified with the <ticking: frequency> annotation.

Instance variables:
	frequency	<Number>	The frequency for the ticking scripts in ticks/sec.!

!CTickingTrigger methodsFor: 'comparing' stamp: 'ar 4/27/2004 21:09'!
= aTrigger
	^super = aTrigger and:[frequency = aTrigger frequency]! !

!CTickingTrigger methodsFor: 'accessing' stamp: 'ar 4/24/2005 04:11'!
eventName
	^#tick! !

!CTickingTrigger methodsFor: 'accessing' stamp: 'ar 4/27/2004 21:04'!
frequency
	^frequency! !

!CTickingTrigger methodsFor: 'accessing' stamp: 'ar 4/27/2004 21:04'!
frequency: aNumber
	frequency := aNumber! !

!CTickingTrigger methodsFor: 'comparing' stamp: 'ar 4/27/2004 21:09'!
hash
	^super hash bitXor: frequency hash! !

!CTickingTrigger methodsFor: 'starting' stamp: 'ar 7/28/2004 22:56'!
installIn: receiver
	"Install the script in the receiver"
	| script |
	script := receiver makeTicking: selector withArguments: #() at: frequency.
	script myTrigger: self.
	^script! !

!CTickingTrigger methodsFor: 'starting' stamp: 'ar 7/28/2004 22:57'!
startUpIn: receiver
	"Install and start this trigger for the given receiver"
	| script |
	script := self installIn: receiver.
	script resumeScript.
	^script! !

Object subclass: #CProtoObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Core-Proto'!

CProtoObject class
	instanceVariableNames: 'classFields methodTriggers classVersionID'!

!CProtoObject commentStamp: 'ar 3/27/2004 15:11' prior: 0!
CProtoObject is the proxy for classes defined in the Tweak-universe (using fields, method triggers, uniclasses, namespaces etc.) and the surrounding "raw" Squeak universe. CProtoObject has therefore only methods dealing with compilation aspects (see the class side) and is otherwise empty.!

CProtoObject class
	instanceVariableNames: 'classFields methodTriggers classVersionID'!

!CProtoObject class methodsFor: 'user interface' stamp: 'ar 5/26/2005 00:28'!
addField: fieldOrName
	| field |
	fieldOrName isString ifTrue:[
		field := CFieldDefinition new.
		field name: fieldOrName asSymbol.
		field beRegularField.
	] ifFalse:[field := fieldOrName].
	(self classFields anySatisfy:[:any| any name = field name]) 
		ifTrue:[^self error: 'Field already defined'].
	field home: self.
	classFields := self classFields copyWith: field.
	field isInstanceVariable ifTrue:[
		self myInstanceVariables: (self myInstanceVariables copyWith: field name).
	].
	field isInstanceVariable ifTrue:[self compileIVarAccessorFor: field name].
	field isVirtualField ifTrue:[self compileVirtualAccessorFor: field name].
	field isRegularField ifTrue:[self compilePropertyAccessorFor: field name].
	self fieldsChanged! !

!CProtoObject class methodsFor: 'compiling' stamp: 'ar 5/4/2005 17:34'!
addSelector: selector withMethod: newMethod
	| oldMethod result done |
	oldMethod := self compiledMethodAt: selector ifAbsent:[nil].
	done := false.
	result := [super addSelector: selector withMethod: newMethod]
		on: CSqueak3Point8Detected do:[:ex| done := true. ex resume].
	done ifFalse:[self methodChanged: selector from: oldMethod to: newMethod].
	^result! !

!CProtoObject class methodsFor: 'compiling' stamp: 'ar 5/4/2005 17:34'!
addSelector: selector withMethod: newMethod notifying: requestor
	| oldMethod result |
	CSqueak3Point8Detected signal. "Only used from 3.8"
	oldMethod := self compiledMethodAt: selector ifAbsent:[nil].
	result := super addSelector: selector withMethod: newMethod notifying: requestor.
	self methodChanged: selector from: oldMethod to: newMethod.
	^result! !

!CProtoObject class methodsFor: 'compiling' stamp: 'ar 5/4/2005 17:34'!
addSelectorSilently: selector withMethod: newMethod
	| oldMethod result done |
	CSqueak3Point8Detected signal. "Only used from 3.8"
	oldMethod := self compiledMethodAt: selector ifAbsent:[nil].
	done := false.
	[result := super addSelectorSilently: selector withMethod: newMethod]
		on: CSqueak3Point8Detected do:[:ex| done := true. ex resume].
	done ifFalse:[self methodChanged: selector from: oldMethod to: newMethod].
	^result! !

!CProtoObject class methodsFor: 'class definition' stamp: 'ar 3/11/2004 19:41'!
allFieldNames
	^Array streamContents:[:s| self allFieldNamesDo:[:f| s nextPut: f]].! !

!CProtoObject class methodsFor: 'class definition' stamp: 'ar 3/11/2004 19:42'!
allFieldNamesDo: aBlock
	| theClass |
	theClass := self.
	[theClass == Object] whileFalse:[
		theClass classFields do:[:f| aBlock value: f name].
		theClass := theClass superclass.
	].! !

!CProtoObject class methodsFor: 'class definition' stamp: 'ar 3/11/2004 19:43'!
allFields
	^Array streamContents:[:s| self allFieldsDo:[:f| s nextPut: f]].! !

!CProtoObject class methodsFor: 'class definition' stamp: 'ar 3/11/2004 19:42'!
allFieldsDo: aBlock
	| theClass |
	theClass := self.
	[theClass == Object] whileFalse:[
		theClass classFields do: aBlock.
		theClass := theClass superclass.
	].! !

!CProtoObject class methodsFor: 'class definition' stamp: 'eem 5/13/2008 09:55'!
allFieldsReverseDo: aBlock
	"Evaluate aBlock with all field definitions, in superclass preceeding subclass order."
	superclass == nil ifFalse:
		[superclass allFieldsReverseDo: aBlock].
	self classFields do: aBlock! !

!CProtoObject class methodsFor: 'user interface' stamp: 'ar 5/6/2004 02:26'!
allReferencesTo: fieldOrName
	| list field |
	fieldOrName isString 
		ifTrue:[field := self fieldNamed: fieldOrName] 
		ifFalse:[field := fieldOrName].
	field ifNil:[^#()].
	list := SortedCollection new.
	self withAllSubclassesDo:[:cls|
		cls allSelect:[:meth| meth refersToField: field] into: list.
	].
	^list! !

!CProtoObject class methodsFor: 'user interface' stamp: 'ar 5/6/2004 02:07'!
allSelect: aBlock into: list
	self selectorsAndMethodsDo:[:sel :meth|
		(aBlock value: meth) ifTrue:[
			list add: (MethodReference new setStandardClass: self methodSymbol: sel).
		].
	].! !

!CProtoObject class methodsFor: 'class hierarchy' stamp: 'ar 3/27/2004 01:52'!
allSubclassesWithLevelDo: classAndLevelBlock startingLevel: level 
	"Walk the tree of subclasses, giving the class and its level"
	| subs |
	classAndLevelBlock value: self value: level.
	self == Class ifTrue:  [^ self].  "Don't visit all the metaclasses"
	"Visit subclasses in alphabetical order"
	subs := SortedCollection sortBlock:[:c1 :c2| c1 name <= c2 name].
	self subclassesDo: [:aClass | subs add: aClass].
	subs do:[:aClass|
		aClass allSubclassesWithLevelDo: classAndLevelBlock
			startingLevel: level+1]! !

!CProtoObject class methodsFor: 'class definition' stamp: 'ar 3/11/2004 19:39'!
allVariableNames
	"Answer all variable names for the receiver"
	^Array streamContents:[:s|
		s nextPutAll: self allInstVarNames.
		s nextPutAll: self allFieldNames.
	].! !

!CProtoObject class methodsFor: 'user interface' stamp: 'ar 5/6/2004 02:24'!
asField: fieldOrName
	fieldOrName isString 
		ifTrue:[^self fieldNamed: fieldOrName] 
		ifFalse:[^fieldOrName].
! !

!CProtoObject class methodsFor: 'monticello' stamp: 'ar 1/22/2005 20:02'!
asMCFieldDefinition
	^((Smalltalk 
		at: #MCTweakFieldsDefinition 
		ifAbsent:[self error: 'Need TweakMC fixes']) new)
			className: self name;
			fields: self instanceVariablesString;
	yourself! !

!CProtoObject class methodsFor: 'uniclasses' stamp: 'ar 3/11/2004 19:21'!
baseClass
	"Answer the base (non-uni) class of the receiver"
	self isUniClass ifTrue:[^superclass] ifFalse:[^self].! !

!CProtoObject class methodsFor: 'compiling' stamp: 'ar 5/4/2005 17:34'!
basicRemoveSelector: selector
	| oldMethod result |
	CSqueak3Point8Detected signal. "Only used in 3.8"
	oldMethod := self compiledMethodAt: selector ifAbsent:[^nil].
	result := super basicRemoveSelector: selector.
	self methodChanged: selector from: oldMethod to: nil.
	^result! !

!CProtoObject class methodsFor: 'uniclasses' stamp: 'ar 6/3/2005 14:35'!
beUniClassFrom: aClass
	"Prepare the receiver to be a uniclass based on aClass' definition."
	| org def |
	"--- copy the organization manually - something is wrong with #copy here---"
	org := organization.	organization := nil.
	self selectors do:[:sel|
		self organization classify: sel under: (org categoryOfElement: sel).
	].
	"--- this should be ClassOrganizer>>copy ---"

	name := '*', aClass baseClass name asString.
	aClass == aClass baseClass ifFalse:[
		"Don't copy this if we're subclassing for real"
		classFields := aClass classFields collect:[:each| 
			def := each shallowCopy.
			def home: self.
			def].
	].
	^self! !

!CProtoObject class methodsFor: 'class definition' stamp: 'ar 3/27/2004 01:56'!
bindingOf: varName
	"Answer the binding of some variable resolved in the scope of the receiver"
	| aSymbol binding |
	aSymbol := varName asSymbol.
	"First look in classVar dictionary."
	binding := self classPool bindingOf: aSymbol.
	binding ifNotNil:[^binding].

	"Next look in shared pools."
	self sharedPools do:[:pool | 
		binding := pool bindingOf: aSymbol.
		binding ifNotNil:[^binding].
	].

	"Next look in declared environment."
	binding := self environment bindingOf: aSymbol.
	binding ifNotNil:[^binding].

	"Finally look higher up the superclass chain and fail at the end."
	superclass == nil ifTrue: [^ nil].
	superclass environment == self environment
		ifFalse:[^nil]. "don't leave environment"
	^superclass bindingOf: aSymbol
! !

!CProtoObject class methodsFor: 'user interface' stamp: 'ar 1/22/2005 17:19'!
browseAllReferencesTo: fieldOrName
	"CPlayer browseAllReferencesTo: #costume"
	^SystemNavigation new
		browseMessageList: (self allReferencesTo: fieldOrName)
		name: fieldOrName name,' references'
		autoSelect: (self asField: fieldOrName) name.! !

!CProtoObject class methodsFor: 'class definition' stamp: 'ar 5/5/2004 23:22'!
browserDefinition: style
	"Answer a definition of the receiver shown in the browser with the given style"

	"Default"
	^self tweakDefinition! !

!CProtoObject class methodsFor: 'accessing' stamp: 'ar 3/27/2004 05:37'!
category
	"Answer the system organization category for the receiver."
	^category ifNil:[self environment organization categoryOfElement: self localName]! !

!CProtoObject class methodsFor: 'compiling' stamp: 'ar 3/25/2004 17:09'!
changeEventFor: aMethod
	"If present, answer the value of the <bewareOf:> annotation in the method specifying which event to listen to for changes of the method."
	| props |
	props := aMethod properties ifNil:[^nil].
	^props at: #bewareOf ifAbsent:[nil].! !

!CProtoObject class methodsFor: 'queries' stamp: 'ar 2/7/2005 00:44'!
chooseInstVarAlphabeticallyThenDo: aBlock
	| allVars index |
	"Put up a menu of all the instance variables in the receiver, presented in alphabetical order, and when the user chooses one, evaluate aBlock with the chosen variable as its parameter."
	allVars := (self allFields collect:[:f| f name]) asSortedArray.
	allVars isEmpty ifTrue: [^ self inform: 'There are no
instance variables'].

	index := UIManager default chooseFrom: allVars lines: #() title: 'Fields in
', self name.
	index = 0 ifTrue: [^ self].
	aBlock value: (allVars at: index)! !

!CProtoObject class methodsFor: 'queries' stamp: 'ar 2/7/2005 00:53'!
chooseInstVarThenDo: aBlock 
	"Put up a menu of all the instance variables in the receiver, and when the user chooses one, evaluate aBlock with the chosen variable as its parameter.  If the list is 6 or larger, then offer an alphabetical formulation as an alternative. triggered by a 'show alphabetically' item at the top of the list."

	| lines labelStream allVars index count offerAlpha fields |
	(count := self allFields size) = 0 
		ifTrue:[^ self inform: 'There are no\instance variables.' withCRs].

	allVars := OrderedCollection new.
	lines := OrderedCollection new.
	labelStream := WriteStream on: Array new.

	(offerAlpha := count > 5) ifTrue:[
		lines add: 1.
		allVars add: 'show alphabetically'.
		labelStream nextPut: allVars first].
	self withAllSuperclasses do:[:class |
		(class includesBehavior: CProtoObject)
			ifTrue:[fields := class classFields]
			ifFalse:[fields := #()].
		fields do:[:f |
			labelStream nextPut: f name.
			allVars add: f name].
		fields isEmpty ifFalse: [lines add: allVars size]].
	(lines size > 0 and: [lines last = allVars size]) ifTrue:
		[lines removeLast].  "dispense with inelegant line beneath last item"
	index := UIManager default chooseFrom: labelStream contents lines: lines title: 'Fields in
', self name.
	index = 0 ifTrue: [^ self].
	(index = 1 and: [offerAlpha]) ifTrue: [^ self chooseInstVarAlphabeticallyThenDo: aBlock].
	aBlock value: (allVars at: index)! !

!CProtoObject class methodsFor: 'monticello' stamp: 'ar 1/22/2005 20:02'!
classDefinitions
	self == CProtoObject ifTrue:[^super classDefinitions].
	^ Array 
			with: self asClassDefinition
			with: self asMCFieldDefinition! !

!CProtoObject class methodsFor: 'class definition' stamp: 'ar 4/28/2005 18:38'!
classFields
	"At some point we need to pull the switch here. We should only use the 'classFields' and have the iVars identify itself as instance variables."
	^classFields ifNil:["Construct from iVars"
		self myInstanceVariables collect:[:each|
			(CFieldDefinition new)
				name: each asSymbol;
				beInstanceVariable;
				yourself]]! !

!CProtoObject class methodsFor: 'queries' stamp: 'sw 5/18/2005 02:00'!
classThatDefinesField: fieldName
	"Answer the class that defines the given field, nil if none."

	self withAllSuperclasses do: [:aClass |
		(aClass includesBehavior: CProtoObject)
			ifTrue:
				[aClass classFields do:
					[:f | f name = fieldName ifTrue: [^ aClass]]]].

	^ nil! !

!CProtoObject class methodsFor: 'queries' stamp: 'sw 5/18/2005 02:03'!
classThatDefinesInstanceVariable: variableName
	"Answer the class that defines the variable, which may be a true inst var or a field"

	(self classThatDefinesField: variableName) ifNotNilDo: [:cl | ^ cl].

	^ super classThatDefinesInstanceVariable: variableName! !

!CProtoObject class methodsFor: 'versions' stamp: 'ar 6/1/2005 11:42'!
classVersionID
	^classVersionID ifNil:[classVersionID := UUID new].! !

!CProtoObject class methodsFor: 'versions' stamp: 'ar 6/1/2005 11:43'!
classVersionID: aUUID
	classVersionID ifNotNil:[^self error: 'Cannot change version ID'].
	classVersionID := aUUID.! !

!CProtoObject class methodsFor: 'compiling' stamp: 'ar 5/18/2005 09:21'!
compileAllFrom: oldClass
	"This is called from ClassBuilder after recreating the class. ClassBuilder doesn't flush the method triggers so we need to make sure we only have those that really ought to be there."
	methodTriggers := nil. "flush triggers"
	^super compileAllFrom: oldClass! !

!CProtoObject class methodsFor: 'compiling' stamp: 'ar 5/6/2004 03:17'!
compileIVarAccessorFor: fieldName
	| code cat |
	cat := (self organization categoryOfElement: fieldName) ifNil:['accessing'].
	code := String streamContents:[:s|
		s nextPutAll: fieldName.
		s crtab; nextPutAll: '"Answer the ', fieldName, ' of the receiver"'.
		s crtab; nextPutAll: '^', fieldName.
	].
	self compile: code classified: cat.
	code := String streamContents:[:s|
		s nextPutAll: fieldName; nextPutAll: ': aValue'.
		s crtab; nextPutAll: '"Modify the receiver''s ', fieldName, '"'.
		s crtab; nextPutAll: fieldName.
		s nextPutAll: ' := aValue'.
	].
	self compile: code classified: cat! !

!CProtoObject class methodsFor: 'compiling' stamp: 'ar 5/6/2004 03:17'!
compilePropertyAccessorFor: fieldName
	| code eventName cat |
	cat := (self organization categoryOfElement: fieldName) ifNil:['accessing'].
	eventName := fieldName,'Changed'.
	code := String streamContents:[:s|
		s nextPutAll: fieldName.
		s crtab; nextPutAll: '"Answer the ', fieldName, ' of the receiver"'.
		s crtab; nextPutAll: '<bewareOf: #', eventName,'>'.
		s crtab; nextPutAll: '^self propertyValueAt: #', fieldName.
	].
	self compile: code classified: cat.
	code := String streamContents:[:s|
		s nextPutAll: fieldName; nextPutAll: ': aValue'.
		s crtab; nextPutAll: '"Modify the receiver''s ', fieldName, '"'.
		s crtab; nextPutAll: '^self propertyValueAt: #', fieldName.
		s nextPutAll: ' put: aValue'.
		s nextPutAll: ' with: #', eventName.
	].
	self compile: code classified: cat.! !

!CProtoObject class methodsFor: 'compiling' stamp: 'ar 5/5/2004 21:41'!
compileVirtualAccessorFor: fieldName
	| code cat |
	cat := (self organization categoryOfElement: fieldName) ifNil:['accessing'].
	code := String streamContents:[:s|
		s nextPutAll: fieldName.
		s crtab; nextPutAll: '"Answer the ', fieldName, ' of the receiver"'.
		s crtab; nextPutAll: '^self mustBeImplemented: #', fieldName.
	].
	self compile: code classified: cat.
	code := String streamContents:[:s|
		s nextPutAll: fieldName; nextPutAll: ': aValue'.
		s crtab; nextPutAll: '"Modify the receiver''s ', fieldName, '"'.
		s crtab; nextPutAll: '^self mustBeImplemented: #', fieldName,':'.
	].
	self compile: code classified: cat.! !

!CProtoObject class methodsFor: 'compiler' stamp: 'ar 3/26/2004 16:42'!
compilerClass
	"Answer a compiler class appropriate for source methods of this class."
	^self == CProtoObject ifTrue:[Compiler] ifFalse:[CCompiler]! !

!CProtoObject class methodsFor: 'method triggers' stamp: 'ar 5/7/2005 16:28'!
computeMethodTriggers
	"Compute the method triggers defined in the receiver"
	| trigger |
	"Inherited triggers go first"
	methodTriggers := superclass methodTriggers.
	self selectorsAndMethodsDo:[:selector :method|
		trigger := self methodTriggerFor: method selector: selector.
		trigger ifNotNil:[
			"Filter out overrides"
			methodTriggers := methodTriggers reject:[:trig| trig selector == selector].
			"Add new trigger"
			methodTriggers := methodTriggers copyWith: trigger.
		].
	].
	^methodTriggers! !

!CProtoObject class methodsFor: 'user interface' stamp: 'ar 5/6/2004 15:34'!
defineField: fieldDef colorName: cc
	cc = #none 
		ifTrue:[fieldDef color: nil]
		ifFalse:[fieldDef color: (Color perform: cc)].
	self fieldsChanged.! !

!CProtoObject class methodsFor: 'class definition' stamp: 'ar 5/26/2005 00:30'!
defineFields: aString
	"Give us a new set of field names"
	| newFields oldFields field myVars root list |
	(aString beginsWith: '<?xml' caseSensitive: false) ifTrue:[
		"XML field definition"
		root := (XMLDOMParser parseDocumentFrom: aString readStream) elements first.
		root name == #fields ifFalse:[^self error:'Not a field definition'].
		classFields := root elements asArray collect:[:node|
			field := CFieldDefinition new.
			field importXmlFrom: node.
			field home: self.
			field].
		self myInstanceVariables: (classFields 
			select:[:each| each isInstanceVariable]
			thenCollect:[:each| each name]) asArray.
		Utilities informUserDuring:[:bar| 
			self withAllSubclassesDo:[:cls| bar value: 'Recompiling ', cls name. cls compileAll]
		].
		^self changed].

	newFields := (Scanner new scanFieldNames: aString) collect: [:x | x asSymbol].
	oldFields := Dictionary new.
	self classFields do:[:f| oldFields at: f name put: f].
	classFields := newFields collect:[:fieldName|
		field := oldFields removeKey: fieldName ifAbsent:[nil].
		field ifNil:["Make us a new one"
			field := CFieldDefinition new.
			field name: fieldName.
			field beRegularField.
			field home: self.
			self compilePropertyAccessorFor: field name.
		].
		field.
	].
	oldFields := oldFields asArray.
	oldFields isEmpty ifTrue:[
		self myInstanceVariables: (classFields 
			select:[:each| each isInstanceVariable]
			thenCollect:[:each| each name]) asArray.
		Utilities informUserDuring:[:bar| 
			self withAllSubclassesDo:[:cls| bar value: 'Recompiling ', cls name. cls compileAll]
		].
		^self fieldsChanged].
	"Remove remaining fields"
	myVars := self myInstanceVariables.
	oldFields do:[:each|
		each isInstanceVariable 
			ifTrue:[myVars := myVars copyWithout: each name].
		each toGet ifNotNil:[self removeSelector: each toGet].
		each toSet ifNotNil:[self removeSelector: each toSet].
	].
	self myInstanceVariables: myVars.
	Utilities informUserDuring:[:bar| 
		self withAllSubclassesDo:[:cls| bar value: 'Recompiling ', cls name. cls compileAll]
	].
	self fieldsChanged.

	"Figure out obsolete references"
	list := SortedCollection new.
	self allSelect:[:meth| oldFields anySatisfy:[:f| meth refersToField: f]] into: list.
	list size > 0 ifTrue:[
		SystemNavigation new browseMessageList: list name: 'Obsolete References'
	].
! !

!CProtoObject class methodsFor: 'user interface' stamp: 'ar 5/6/2004 00:31'!
defineIVarField: fieldDef
	"Make fieldDef be an instance variable"
	fieldDef isInstanceVariable ifTrue:[^self].

	fieldDef beInstanceVariable.
	self myInstanceVariables: (self myInstanceVariables copyWith: fieldDef name).

	"Compile iVar accessors"
	self compileIVarAccessorFor: fieldDef name.
	self fieldsChanged.! !

!CProtoObject class methodsFor: 'user interface' stamp: 'ar 5/6/2004 00:01'!
defineRegularField: fieldDef
	"Make fieldDef be a regular field"
	| iVar |
	fieldDef isRegularField ifTrue:[^self].
	"First compile property accessor"
	self compilePropertyAccessorFor: fieldDef name.
	iVar := fieldDef isInstanceVariable.
	fieldDef beRegularField.
	iVar ifTrue:[
		self myInstanceVariables: (self myInstanceVariables copyWithout: fieldDef name)
	].
	self fieldsChanged.! !

!CProtoObject class methodsFor: 'user interface' stamp: 'ar 5/6/2004 00:01'!
defineVirtualField: fieldDef
	| iVar |
	fieldDef isVirtualField ifTrue:[^self].
	"First compile virtual accessor"
	self compileVirtualAccessorFor: fieldDef name.
	iVar := fieldDef isInstanceVariable.
	fieldDef beVirtualField.
	iVar ifTrue:[
		self myInstanceVariables: (self myInstanceVariables copyWithout: fieldDef name).
	].
	self fieldsChanged.! !

!CProtoObject class methodsFor: 'compiling' stamp: 'ar 3/11/2004 19:17'!
deserialize: fieldName using: aSelectorOrNil
	"Associate aSelectorOrNil as the serializer for the given field"
	self classFields do:[:aField|
		aField name = fieldName ifTrue:[
			aField deserializer: aSelectorOrNil.
		].
	].
	"We should probably raise an exception here if the field isn't even present..."! !

!CProtoObject class methodsFor: 'compiler' stamp: 'ar 3/26/2004 21:29'!
evaluatorClass
	"Answer an evaluator class appropriate for evaluating expressions in the  context of this class."
	^self == CProtoObject ifTrue:[Compiler] ifFalse:[CCompiler]! !

!CProtoObject class methodsFor: 'user interface' stamp: 'ar 1/22/2005 17:40'!
fieldMenuFor: fieldDef
	"Answer an interactive menu for the given field definition"
	| menu item colorMenu theColor |
	Processor activeProcess isScriptingProcess ifFalse:[^self morphicFieldMenuFor: fieldDef].

	menu := CWidgetLibrary default menuClass new.
	menu label: fieldDef name.

	item := menu add: 'Regular Field' target: self 
				action: #defineRegularField: argument: fieldDef.
	item checked: fieldDef isRegularField.
	item := menu add: 'Virtual Field' target: self 
				action: #defineVirtualField: argument: fieldDef.
	item checked: fieldDef isVirtualField.
	item := menu add: 'Instance Variable' target: self 
				action: #defineIVarField: argument: fieldDef.
	item checked: fieldDef isInstanceVariable.

	menu addSeparator.

	item := menu add: 'Serializable' target: self 
				action: #flipSerializable: argument: fieldDef.
	item checked: fieldDef isSerializable.
	fieldDef isVirtualField ifTrue:[item disabled: true].

	menu addSeparator.
	colorMenu := CWidgetLibrary default menuClass new.
	colorMenu label: 'Color'.
	#(none red green blue yellow magenta cyan) do:[:cc|
		theColor := cc == #none ifTrue:[nil] ifFalse:[Color perform: cc].
		item := colorMenu add: cc target: self action: #defineField:colorName: arguments: {fieldDef. cc}.
		item checked: fieldDef color = theColor.
	].
	menu add: 'Field Color' subMenu: colorMenu.

	^menu! !

!CProtoObject class methodsFor: 'compiling' stamp: 'eem 5/12/2008 15:06'!
fieldVariableNodes
	"Answer the variable nodes for our fields"
	^self allFields collect:[:each| FieldNode new fieldDefinition: each].! !

!CProtoObject class methodsFor: 'class definition' stamp: 'ar 6/1/2005 11:43'!
fieldsChanged
	ChangeSet current changeClass: self from: self.
	self flushClassVersionID.
	self signal: #fieldsChanged.
! !

!CProtoObject class methodsFor: 'compiling' stamp: 'ar 3/25/2004 17:17'!
fixChangeEvents
	"CProtoObject fixChangeEvents"
	| evt theField |
	self withAllSubclassesDo:[:each|
		each selectorsAndMethodsDo:[:sel :meth|
			evt := self changeEventFor: meth.
			evt ifNotNil:[
				theField := each classFields detect:[:any| any name = sel] ifNone:[nil].
				theField ifNotNil:[theField changeEvent: evt]]]].! !

!CProtoObject class methodsFor: 'deprecated' stamp: 'ar 5/26/2005 00:33'!
fixHome "CProtoObject fixHome"
	"Fix home bindings"
	self flushMethodTriggers.
	self classFields do:[:each| each home: self]. 		"myself"
	self subclasses do:[:each| each fixHome].			"subclasses"
	self hiddenSubclasses do:[:each| each fixHome].	"uniclasses"! !

!CProtoObject class methodsFor: 'user interface' stamp: 'ar 5/6/2004 00:01'!
flipSerializable: fieldDef
	fieldDef serializable: fieldDef serializable not.
	self fieldsChanged.! !

!CProtoObject class methodsFor: 'versions' stamp: 'ar 6/1/2005 11:42'!
flushClassVersionID
	classVersionID := nil.
	self subclassesDo:[:each| each flushClassVersionID].
	self hiddenSubclasses do:[:each| each flushClassVersionID].! !

!CProtoObject class methodsFor: 'method triggers' stamp: 'ar 5/8/2005 00:04'!
flushMethodTriggers
	"Flush method triggers for the receiver"
	methodTriggers ifNil:[^self].
	methodTriggers := nil.
	self subclassesDo:[:each| each flushMethodTriggers].
	self hiddenSubclasses do:[:each| each flushMethodTriggers].

! !

!CProtoObject class methodsFor: 'uniclasses' stamp: 'ar 3/11/2004 19:19'!
instVarNamesFrom: baseClass
	| aClass ivNames |
	ivNames := #().
	aClass := self.
	[aClass == baseClass] whileFalse:[
		ivNames := aClass instVarNames, ivNames.
		aClass := aClass superclass].
	^ivNames! !

!CProtoObject class methodsFor: 'class definition' stamp: 'bf 4/25/2005 11:47'!
instanceVariablesString
	"Fake it."
	| writer |
	self == CProtoObject ifTrue:[^super instanceVariablesString].
	^String streamContents:[:s| 
		writer := XMLWriterIndenting on: s.
		writer xmlDeclaration: '1.0'.
		writer startTag: 'fields'; endTag.
		self classFields do:[:f| f storeXmlOn: writer].
		writer endTag: 'fields'.
	].! !

!CProtoObject class methodsFor: 'uniclasses' stamp: 'ar 3/11/2004 19:19'!
isSystemDefined
	^Smalltalk includesKey: name! !

!CProtoObject class methodsFor: 'uniclasses' stamp: 'ar 3/11/2004 19:19'!
isUniClass
	^name first = $*! !

!CProtoObject class methodsFor: 'compiling' stamp: 'ar 5/4/2005 17:33'!
methodChanged: selector from: oldMethod to: newMethod
	"The method with the given selector has changed:
		- if (oldMethod == nil) then the method was newly added
		- if (newMethod == nil) then the method was removed
		- otherwise, the method was recompiled.
	"
	^self updatePropertiesFor: selector from: oldMethod to: newMethod! !

!CProtoObject class methodsFor: 'method triggers' stamp: 'ar 4/27/2004 21:02'!
methodTriggerAt: selector
	"Answer the method trigger for the given selector, if any"
	| method |
	method := self compiledMethodAt: selector ifAbsent:[^nil].
	^self methodTriggerFor: method selector: selector! !

!CProtoObject class methodsFor: 'method triggers' stamp: 'ar 5/26/2005 00:26'!
methodTriggerFor: method selector: methodSelector
	"Answer the method trigger for the given selector, if any"
	| props value |
	props := method properties ifNil:[^nil].
	value := props at: #on ifAbsent:[nil].
	value ifNotNil:[
		^(CEventTrigger new)
			home: self;
			selector: methodSelector;
			eventName: value;
			yourself].
	value := props at: #ticking ifAbsent:[nil].
	value ifNotNil:[
		^(CTickingTrigger new)
			home: self;
			selector: methodSelector;
			frequency: value;
			yourself].
	value := props at: #on:in: ifAbsent:[nil].
	value ifNotNil:[
		^(CEventTrigger new)
			home: self;
			selector: methodSelector;
			eventName: value first;
			field: (self fieldNamed: value last);
			yourself].
	^nil! !

!CProtoObject class methodsFor: 'method triggers' stamp: 'ar 5/7/2005 16:28'!
methodTriggers
	"Answer the method triggers defined in the receiver"
	^methodTriggers ifNil:[self computeMethodTriggers]! !

!CProtoObject class methodsFor: 'user interface' stamp: 'ar 1/22/2005 17:22'!
morphicFieldMenuFor: fieldDef
	"Answer an interactive menu for the given field definition"
	| menu str theColor colorMenu menuMorph |
	menuMorph := Smalltalk at: #MenuMorph ifAbsent:[^nil].
	menu := menuMorph new.
	menu title: fieldDef name.
	str := (fieldDef isRegularField ifTrue:['<on>'] ifFalse:['<off>']), 'Regular Field'.
	menu add: str target: self selector: #defineRegularField: argument: fieldDef.
	str := (fieldDef isVirtualField ifTrue:['<on>'] ifFalse:['<off>']), 'Virtual Field'.
	menu add: str target: self selector: #defineVirtualField: argument: fieldDef.
	str := (fieldDef isInstanceVariable ifTrue:['<on>'] ifFalse:['<off>']), 'Instance Variable'.
	menu add: str target: self selector: #defineIVarField: argument: fieldDef.

	fieldDef isVirtualField ifFalse:[
		menu addLine.
		str := (fieldDef isSerializable ifTrue:['<on>'] ifFalse:['<off>']), 'Serializable'.
		menu add: str target: self selector: #flipSerializable: argument: fieldDef.
		menu addLine.
	].
	colorMenu := menuMorph new.
	#(none red green blue yellow magenta cyan) do:[:cc|
		theColor := cc == #none ifTrue:[nil] ifFalse:[Color perform: cc].
		str := ((fieldDef color = theColor) ifTrue:['<on>'] ifFalse:['<off>']), cc.
		colorMenu add: str target: self selector: #defineField:colorName: argumentList: {fieldDef. cc}.
	].
	menu add: 'Field Color' subMenu: colorMenu.
	^menu! !

!CProtoObject class methodsFor: 'compiling' stamp: 'ar 5/5/2004 21:37'!
myInstanceVariables
	^instanceVariables ifNil:[#()]! !

!CProtoObject class methodsFor: 'compiling' stamp: 'ar 5/5/2004 22:43'!
myInstanceVariables: anArray
	self instVarNames = anArray ifTrue:[^self].
	(ClassBuilder new)
		superclass: superclass
		subclass: name
		instanceVariableNames: anArray
		classVariableNames: self classVariablesString
		poolDictionaries: self sharedPoolsString
		category: self category! !

!CProtoObject class methodsFor: 'printing' stamp: 'tak 5/26/2005 16:36'!
nameForDisplay
	| n | 
	n := super nameForDisplay.
	n first = $* ifTrue: [n := n allButFirst].
	(n first = $C and: [n size > 1 and: [n second isUppercase]])
		ifTrue: [n := n allButFirst].
	(n size > 8 and:[n endsWith: 'Player']) ifTrue:[
		n := n allButLast: 6.
	].
	^n translated! !

!CProtoObject class methodsFor: 'versions' stamp: 'ar 6/1/2005 11:42'!
newClassVersionID
	classVersionID := UUID new.
	self subclassesDo:[:each| each newClassVersionID].
	self hiddenSubclasses do:[:each| each newClassVersionID].! !

!CProtoObject class methodsFor: 'uniclasses' stamp: 'ar 3/27/2004 14:52'!
newUniClass
	"Make a new uni class just like me"
	| baseClass newClass |
	"Get the base (non-uni) class"
	baseClass := self baseClass.
	"Create a new anonymous subclass"
	newClass := ClassBuilder new
		newSubclassOf: baseClass
		type: baseClass typeOfClass
		instanceVariables: (self instVarNamesFrom: baseClass)
		from: (self == baseClass ifFalse:[self]).
	baseClass addUniClass: newClass.
	baseClass removeSubclass: newClass.
	newClass beUniClassFrom: self.
	^newClass! !

!CProtoObject class methodsFor: 'compiler' stamp: 'ar 3/26/2004 16:42'!
parserClass
	"Answer a compiler class appropriate for source methods of this class."
	^self == CProtoObject ifTrue:[Parser] ifFalse:[CParser]! !

!CProtoObject class methodsFor: 'class definition' stamp: 'eem 1/28/2009 16:03'!
printMethodChunk: selector withPreamble: doPreamble on: outStream
		moveSource: moveSource toFile: fileIndex
	"Copy the source code for the method associated with selector onto the fileStream.  If moveSource true, then also set the source code pointer of the method."
	| preamble method oldPos newPos sourceFile endPos |
	doPreamble 
		ifTrue: [preamble := self printString , ' methodsFor: ' ,
					(self organization categoryOfElement: selector) asString printString]
		ifFalse: [preamble := ''].
	method := self methodDict at: selector ifAbsent:
		[outStream nextPutAll: selector; cr.
		outStream tab; nextChunkPut: '** ERROR!!  THIS SCRIPT IS MISSING ** ' translated; cr; cr.
		outStream nextPutAll: '  '.
		^ outStream].

	((method fileIndex = 0
		or: [(SourceFiles at: method fileIndex) == nil])
		or: [(oldPos := method filePosition) = 0])
		ifTrue:
		["The source code is not accessible.  We must decompile..."
		preamble size > 0 ifTrue: [outStream cr; nextPut: $!!; nextChunkPut: preamble; cr].
		outStream nextChunkPut: (self decompilerClass new decompile: selector
											in: self method: method) decompileString]
		ifFalse:
		[sourceFile := SourceFiles at: method fileIndex.
		preamble size > 0
			ifTrue:    "Copy the preamble"
				[outStream copyPreamble: preamble from: sourceFile at: oldPos]
			ifFalse:
				[sourceFile position: oldPos].
		"Copy the method chunk"
		newPos := outStream position.
		outStream copyMethodChunkFrom: sourceFile.
		sourceFile skipSeparators.      "The following chunk may have ]style["
		sourceFile peek == $] ifTrue: [
			outStream cr; copyMethodChunkFrom: sourceFile].
		moveSource ifTrue:    "Set the new method source pointer"
			[endPos := outStream position.
			method checkOKToAdd: endPos - newPos at: newPos.
			method setSourcePosition: newPos inFile: fileIndex]].
	preamble size > 0 ifTrue: [outStream nextChunkPut: ' '].
	^ outStream cr! !

!CProtoObject class methodsFor: 'printing' stamp: 'ar 3/27/2004 01:36'!
printOn: aStream 
	self environment scopeFor: self name from: nil envtAndPathIfFound:[:env :path|
		aStream nextPutAll: path.
	].
	aStream nextPutAll: self name.! !

!CProtoObject class methodsFor: 'compiling' stamp: 'ar 4/27/2004 21:16'!
recreateMethodProperties
	"Recreate all the method properties of the receiver the hard way"
	self selectorsAndMethodsDo:[:selector :method|
		self updatePropertiesFor: selector from: method to: nil.
		self updatePropertiesFor: selector from: nil to: method.
	].! !

!CProtoObject class methodsFor: 'user interface' stamp: 'pe 12/9/2005 14:16'!
removeField: fieldOrName
	| fieldName field |
	fieldName := fieldOrName isString ifTrue:[fieldOrName] ifFalse:[fieldOrName name].
	field := classFields detect:[:any| any name = fieldName] ifNone:[nil].
	field ifNil:[^nil].
	classFields := classFields copyWithout: field.
	field toGet ifNotNil:[self removeSelector: field toGet].
	field toSet ifNotNil:[self removeSelector: field toSet].
	field isInstanceVariable ifTrue:[
		self myInstanceVariables: (self myInstanceVariables copyWithout: fieldName).
	].
	self fieldsChanged! !

!CProtoObject class methodsFor: 'compiling' stamp: 'ar 5/4/2005 17:34'!
removeSelector: selector
	| oldMethod result done |
	oldMethod := self compiledMethodAt: selector ifAbsent:[^nil].
	done := false.
	result := [super removeSelector: selector] 
		on: CSqueak3Point8Detected do:[:ex| done := true. ex resume].
	done ifFalse:[self methodChanged: selector from: oldMethod to: nil].
	^result! !

!CProtoObject class methodsFor: 'class definition' stamp: 'ar 3/11/2004 19:13'!
scopeHas: varName ifTrue: aBlock
	(self bindingOf: varName) ifNotNilDo:[:assoc| aBlock value: assoc. ^true].
	^false! !

!CProtoObject class methodsFor: 'compiling' stamp: 'ar 3/11/2004 19:16'!
serialize: fieldName using: aSelectorOrNil
	"Associate aSelectorOrNil as the serializer for the given field"
	self classFields do:[:aField|
		aField name = fieldName ifTrue:[
			aField serializer: aSelectorOrNil.
		].
	].
	"We should probably raise an exception here if the field isn't even present..."! !

!CProtoObject class methodsFor: 'accessing' stamp: 'ar 2/14/2007 11:54'!
shoutParserClass
	^Smalltalk at: #SHTweakParserST80 ifAbsent:[super shoutParserClass]! !

!CProtoObject class methodsFor: 'source code' stamp: 'ar 5/6/2004 02:41'!
sourceCodeAt: selector
	^self sourceCodeAt: selector ifAbsent:[self methodDict errorNotFound: selector]! !

!CProtoObject class methodsFor: 'source code' stamp: 'ar 5/6/2004 03:16'!
sourceCodeAt: selector ifAbsent: aBlock
	| method |
	method := self methodDict at: selector ifAbsent: [^ aBlock value].
	^method getSourceFor: selector in: self! !

!CProtoObject class methodsFor: 'source code' stamp: 'ar 5/6/2004 02:48'!
sourceCodeAt: selector toGet: field
	^String streamContents:[:s|
		s nextPutAll: selector.
		s crtab; nextPutAll: '"Answer the receiver''s ', field name, '"'.
		field isGeneratingEvents ifTrue:[
			s crtab; nextPutAll: '<bewareOf: #', field changeEvent,'>'.
		].
		s crtab nextPutAll: '^', field name.
	].! !

!CProtoObject class methodsFor: 'source code' stamp: 'ar 5/6/2004 02:47'!
sourceCodeAt: selector toSet: field
	| type typeName |
	type := (field type ifNil:[Object]).
	typeName := (type name first isVowel ifTrue: ['an'] ifFalse: ['a']), type name.
	^String streamContents:[:s|
		s nextPutAll: selector.
		s space; nextPutAll: typeName.
		s crtab; nextPutAll: '"Modify the receiver''s ', field name, '"'.
		s crtab nextPutAll: field name, ' := ', typeName
	].! !

!CProtoObject class methodsFor: 'class definition' stamp: 'ar 5/5/2004 23:17'!
squeakInstanceVariablesString
	^super instanceVariablesString! !

!CProtoObject class methodsFor: 'compiling' stamp: 'ar 4/27/2004 21:11'!
startAllScripts: selector triggeredBy: aTrigger
	"Horribly slow ..."
	self allSubInstancesDo:[:inst| aTrigger startUpIn: inst].! !

!CProtoObject class methodsFor: 'startup' stamp: 'ar 6/16/2005 20:35'!
startUpInIslandReader: reader
	self isUniClass ifTrue:[superclass addUniClass: self].! !

!CProtoObject class methodsFor: 'compiling' stamp: 'ar 3/11/2004 19:16'!
stopAllScripts: selector triggeredBy: aTrigger
	"Horribly slow ..."
	self allSubInstancesDo:[:inst|
		inst myScripts do:[:each|
			(each selector == selector and:[each myTrigger = aTrigger]) ifTrue:[
				inst stopScript: each.
			].
		].
	].! !

!CProtoObject class methodsFor: 'deprecated' stamp: 'ar 6/2/2004 16:21'!
subclass: newName fieldNames: fields instanceVariableNames: iVars classVariableNames: cVars poolDictionaries: pools category: cat
	"Old-style definition - convert to XML before going on"
	| ivString ff writer |
	ivString := String streamContents:[:out|
		writer := XMLWriter on: out.
		writer xmlDeclaration: '1.0'.
		writer startTag: 'fields'; endTag.
		(Scanner new scanFieldNames: iVars) do:[:iv|
			ff := CFieldDefinition new.
			ff name: iv.
			ff beInstanceVariable.
			ff storeXmlOn: writer.
		].
		(Scanner new scanFieldNames: fields) do:[:iv|
			ff := CFieldDefinition new.
			ff name: iv asSymbol.
			ff beRegularField.
			ff storeXmlOn: writer.
		].
		writer endTag: 'fields'.
	].
	^self subclass: newName instanceVariableNames: ivString classVariableNames: cVars poolDictionaries: pools category: cat! !

!CProtoObject class methodsFor: 'class definition' stamp: 'ar 4/28/2005 18:31'!
subclass: newName instanceVariableNames: iVars classVariableNames: cVars poolDictionaries: pools category: cat
	| aClass ivList |
	aClass := Smalltalk at: newName ifAbsent:[nil].
	aClass ifNil:[
		"Define w/o instVars"
		aClass := super subclass: newName 
			instanceVariableNames: '' 
			classVariableNames: cVars 
			poolDictionaries: pools 
			category: cat.
	] ifNotNil:[
		"Define w/ existing instVars"
		ivList := (aClass inheritsFrom: CProtoObject)
			ifTrue:[aClass squeakInstanceVariablesString]
			ifFalse:[aClass instanceVariablesString].
		aClass := super subclass: newName 
			instanceVariableNames: ivList
			classVariableNames: cVars 
			poolDictionaries: pools 
			category: cat.
	].
	"Define for real"
	aClass ifNotNil:[aClass defineFields: iVars].
	^aClass! !

!CProtoObject class methodsFor: 'compiler' stamp: 'ar 3/27/2004 02:07'!
subclassDefinerClass
	"Answer an evaluator class appropriate for evaluating definitions of new 
	subclasses of this class."
	^CCompiler! !

!CProtoObject class methodsFor: 'class definition' stamp: 'ar 5/6/2004 00:39'!
tweakDefinition
	"Answer a definition of the receiver."
	| aStream path |
	aStream := CTextStream on: (Text new: 300).
	superclass == nil
		ifTrue: [aStream nextPutAll: 'ProtoObject']
		ifFalse: [path := ''.
				self environment scopeFor: superclass name from: nil
						envtAndPathIfFound: [:envt :remotePath | path := remotePath].
				aStream nextPutAll: path , superclass name].
	aStream nextPutAll: self kindOfSubclass;
			store: self name.
	aStream cr; tab; nextPutAll: 'instanceVariableNames: '''.
		self classFields do:[:f| 
			aStream withAttribute: (CFieldDefinitionTextAttribute forField: f in: self)
				do:[aStream nextPutAll: (f name ifNil:['???'])].
		] separatedBy:[aStream space].
	aStream nextPut:$'.
	aStream cr; tab; nextPutAll: 'classVariableNames: ';
			store: self classVariablesString.
	aStream cr; tab; nextPutAll: 'poolDictionaries: ';
			store: self sharedPoolsString.
	aStream cr; tab; nextPutAll: 'category: ';
			store: (SystemOrganization categoryOfElement: self name) asString.

	superclass ifNil: [ 
		aStream nextPutAll: '.'; cr.
		aStream nextPutAll: self name.
		aStream space; nextPutAll: 'superclass: nil'. ].

	^ aStream contents! !

!CProtoObject class methodsFor: 'class definition' stamp: 'ar 4/28/2005 18:33'!
updateInstances: oldInstances from: oldClass isMeta: isMeta
	| itsVars myVars varToField fieldToVar map variable instSize newInstances old new |
	isMeta ifTrue: [self error:'Something is horribly wrong here'].
	itsVars := oldClass allInstVarNames.
	myVars := self allInstVarNames.
	itsVars = myVars ifTrue:[^super updateInstances: oldInstances from: oldClass isMeta: isMeta].

	"Figure out the iVars that have been removed and possibly become fields"
	varToField := Dictionary new.
	"Add candidates by populating the dictionary with the fields of the new class"
	self allFieldsDo:[:f| f isInstanceVariable ifFalse:[varToField at: f name put: nil]].
	"Add all vars from old class"
	itsVars keysAndValuesDo:[:slotIndex :slotName| 
		(varToField includesKey: slotName) ifTrue:[varToField at: slotName put: slotIndex].
	].
	"And remove vars still here"
	myVars do:[:slotName| varToField removeKey: slotName ifAbsent:[]].
	varToField := varToField reject:[:slotIndex| slotIndex = nil].

	"Figure out the iVars that have been added and possibly were fields before"
	fieldToVar := Dictionary new.
	(oldClass includesBehavior: CProtoObject) ifTrue:["Not for non-CObjs"
		"All vars from myself class"
		myVars keysAndValuesDo:[:slotIndex :slotName| fieldToVar at: slotName put: slotIndex].
		"And remove vars still there"
		itsVars do:[:slotName| fieldToVar removeKey: slotName ifAbsent:[]].
	].

	map := self instVarMappingFrom: oldClass.
	variable := self isVariable.
	instSize := self instSize.
	newInstances := Array new: oldInstances size.
	1 to: oldInstances size do:[:i|
		old := oldInstances at: i.
		new := self newInstanceFrom: old variable: variable size: instSize map: map.
		fieldToVar keysAndValuesDo:[:slotName :slotIndex| 
			new instVarAt: slotIndex put: (old propertyValueAt: slotName)
		].
		varToField keysAndValuesDo:[:slotName :slotIndex| 
			new propertyValueAt: slotName put: (old instVarAt: slotIndex)
		].
		newInstances at: i put: new.
	].
	"Now perform a bulk mutation of old instances into new ones"
	oldInstances elementsExchangeIdentityWith: newInstances.
	^newInstances "which are now old"! !

!CProtoObject class methodsFor: 'compiling' stamp: 'ar 5/7/2005 16:27'!
updatePropertiesFor: selector from: oldMethod to: newMethod
	| props value fieldName oldTrigger oldEvent newTrigger newEvent theField |
	oldMethod ifNotNil:[
		oldTrigger := self methodTriggerFor: oldMethod selector: selector.
		oldEvent := self changeEventFor: oldMethod.

		"See if the selector defined a getter for one of our fields"
		(newMethod == nil and:[oldMethod numArgs = 0]) ifTrue:[
			self classFields do:[:each|  each toGet == selector ifTrue:[each toGet: nil]].
			"And we really *should* signal the user's attention towards use of that setter"
		].

		"See if the selector defined a setter for one of our fields"
		(newMethod == nil and:[selector last = $: and:[oldMethod numArgs = 1]]) ifTrue:[
			self classFields do:[:each|  each toSet == selector ifTrue:[each toSet: nil]].
			"And we really *should* signal the user's attention towards use of that setter"
		].

		"See if the old method defined a serializer/deserializer"
		props := oldMethod properties.
		props ifNotNil:[
			value := props at: #serialize ifAbsent:[nil].
			value ifNotNil:[self serialize: value using: nil].
			value := props at: #deserialize ifAbsent:[nil].
			value ifNotNil:[self deserialize: value using: nil].
		].
	].

	newMethod ifNotNil:[
		newTrigger := self methodTriggerFor: newMethod selector: selector.
		newEvent := self changeEventFor: newMethod.

		"See if the new method defines a getter for one of our fields"
		(oldMethod  == nil and:[newMethod numArgs = 0]) ifTrue:[
			fieldName := selector.
			self classFields do:[:each| each name = fieldName ifTrue:[each toGet: selector]].
		].

		"See if the new method defines a setter for one of our fields"
		(oldMethod  == nil and:[selector last = $: and:[newMethod numArgs = 1]]) ifTrue:[
			fieldName := selector allButLast.
			self classFields do:[:each| each name = fieldName ifTrue:[each toSet: selector]].
		].

		"See if the new method defines a serializer/deserializer"
		props := newMethod properties.
		props ifNotNil:[
			value := props at: #serialize ifAbsent:[nil].
			value ifNotNil:[self serialize: value using: selector].
			value := props at: #deserialize ifAbsent:[nil].
			value ifNotNil:[self deserialize: value using: selector].
		].
	].

	oldTrigger = newTrigger ifFalse:[
		self flushMethodTriggers.
		oldTrigger ifNotNil:[self stopAllScripts: selector triggeredBy: oldTrigger].
		newTrigger ifNotNil:[self startAllScripts: selector triggeredBy: newTrigger].
	].
	oldEvent = newEvent ifFalse:[
		theField := self classFields detect:[:any| any name = selector] ifNone:[nil].
		theField ifNotNil:[theField changeEvent: newEvent].
	].! !

!CProtoObject class methodsFor: 'compiling' stamp: 'eem 6/19/2008 09:08'!
variablesAndOffsetsDo: aBinaryBlock
	"This is the interface between the compiler and a class's instance or field names.  The
	 class should enumerate aBinaryBlock with the field definitions (with nil offsets) followed
	 by the instance variable name strings and their integer offsets (1-relative).  The order is
	 important; names evaluated later will override the same names occurring earlier."

	self allFieldsReverseDo: [:field| aBinaryBlock value: field value: nil].
	self instVarNamesAndOffsetsDo: aBinaryBlock! !

!CProtoObject class methodsFor: 'queries' stamp: 'ar 2/7/2005 00:47'!
whichSelectorsAccess: instVarName
	"Answer a Set of selectors whose methods access the argument, 
	instVarName, as a named instance variable."
	| fieldDef meth messageA messageB set |
	set := super whichSelectorsAccess: instVarName.
	fieldDef := self allFields detect:[:any| any name = instVarName] ifNone:[^set].
	messageA := fieldDef toGet.
	messageB := fieldDef toSet.
	self methodDict keysDo:[:sel | 
		meth := self methodDict at: sel.
		"very ugly ... and VERY fast"
		((meth pointsTo: messageA) or:[meth pointsTo: messageB])
			ifTrue:[set add: sel].
	].
	^set
	"CPlayer whichSelectorsAccess: 'borderWidth'."! !

!CProtoObject class methodsFor: 'queries' stamp: 'ar 5/13/2005 11:31'!
whichSelectorsStoreInto: instVarName 
	"Answer a Set of selectors whose methods access the argument, 
	instVarName, as a named instance variable."
	| fieldDef meth set instVarIndex |
	instVarIndex := self allInstVarNames indexOf: instVarName ifAbsent: [0].
	instVarIndex = 0 ifFalse:[
		^self methodDict keys select:[:sel | (self methodDict at: sel) writesField: instVarIndex]
	].
	fieldDef := self allFields detect:[:any| any name = instVarName] ifNone:[^Set new].
	set := Set new.
	self methodDict keysDo:[:sel | 
		meth := self methodDict at: sel.
		"very ugly ... and VERY fast"
		(meth pointsTo: fieldDef toSet) ifTrue:[set add: sel]].
	^set
	"CPlayer whichSelectorsStoreInto: 'borderWidth'."! !

!CProtoObject class methodsFor: 'method triggers' stamp: 'ar 4/27/2004 21:18'!
zapMethodTriggers
	"Zap the existing method triggers - DON'T DO THIS!!!!!!"
	methodTriggers := nil.
! !

Object subclass: #CWidgetLibrary
	instanceVariableNames: ''
	classVariableNames: 'Default'
	poolDictionaries: ''
	category: 'Tweak-Core-Proto'!

!CWidgetLibrary commentStamp: 'ar 3/27/2004 18:01' prior: 0!
CWidgetLibrary defines which default costumes to use for various players, the color scheme and some other UI aspects.!

!CWidgetLibrary class methodsFor: 'accessing' stamp: 'ar 2/29/2004 18:33'!
default
	^Default ifNil:[Default := self new].! !

!CWidgetLibrary class methodsFor: 'accessing' stamp: 'ar 2/29/2004 18:33'!
default: aLibrary
	Default := aLibrary.! !

!CWidgetLibrary methodsFor: 'basic' stamp: 'bgf 7/11/2009 15:38'!
balloonFont
	^ BalloonMorph balloonFont	"Where preferences puts it."! !

!CWidgetLibrary methodsFor: 'colors' stamp: 'ar 1/22/2005 15:29'!
baseColor
	"The base color for this scheme"
	^self subclassResponsibility! !

!CWidgetLibrary methodsFor: 'buttons' stamp: 'ar 1/22/2005 15:32'!
buttonCostume
	^nil! !

!CWidgetLibrary methodsFor: 'buttons' stamp: 'ar 1/22/2005 15:32'!
checkBoxCostume
	^nil! !

!CWidgetLibrary methodsFor: 'lists' stamp: 'ar 1/22/2005 15:31'!
dropDownListCostume
	^nil! !

!CWidgetLibrary methodsFor: 'colors' stamp: 'ar 1/22/2005 15:29'!
frameColor
	"This is the color to be used for 1px borders (framing)"
	^self subclassResponsibility! !

!CWidgetLibrary methodsFor: 'colors' stamp: 'ar 3/15/2004 00:00'!
gradientColors
	"This describes the colors to use for any gradients"
	^{Color white. self baseColor}! !

!CWidgetLibrary methodsFor: 'colors' stamp: 'ar 8/20/2007 12:28'!
insertionPointColor
	"Answer the color to be used for the insertion point"
	^self subclassResponsibility! !

!CWidgetLibrary methodsFor: 'basic' stamp: 'ar 1/22/2005 15:31'!
labelCostume
	^nil! !

!CWidgetLibrary methodsFor: 'lists' stamp: 'ar 1/22/2005 15:31'!
listCostume
	^nil! !

!CWidgetLibrary methodsFor: 'menus' stamp: 'ar 1/22/2005 17:20'!
menuClass
	^self subclassResponsibility! !

!CWidgetLibrary methodsFor: 'colors' stamp: 'bgf 9/13/2008 19:06'!
menuColor
	"Answer the base color for menus (pulldowns and popups)"
	^ Color white! !

!CWidgetLibrary methodsFor: 'colors' stamp: 'ar 3/15/2004 00:07'!
menuColors
	"Answer the gradient to be used for menu backgrounds"
	^{Color white. self paneColor}! !

!CWidgetLibrary methodsFor: 'menus' stamp: 'ar 1/22/2005 15:31'!
menuCostume
	^nil! !

!CWidgetLibrary methodsFor: 'colors' stamp: 'bgf 8/25/2008 09:53'!
menuFont
	"Answer the font to be used for menu items"
	^ self subclassResponsibility! !

!CWidgetLibrary methodsFor: 'colors' stamp: 'bgf 8/24/2007 23:49'!
menuFrameColor
	"This is the color to be used for 1px borders around menus."
	^self subclassResponsibility! !

!CWidgetLibrary methodsFor: 'menus' stamp: 'ar 1/22/2005 15:31'!
menuItemCostume
	^nil! !

!CWidgetLibrary methodsFor: 'buttons' stamp: 'bgf 1/10/2009 09:51'!
mouseOverHighlightForButton: aButton
	"Return a CCostumeHighlight used as rollover-feedback for enabled push buttons."
	self subclassResponsibility.! !

!CWidgetLibrary methodsFor: 'colors' stamp: 'ar 3/14/2004 23:57'!
noColor
	"noColor exists only to describe that we *mean* to use a symbolic color from the library"
	^Color transparent! !

!CWidgetLibrary methodsFor: 'colors' stamp: 'ar 1/22/2005 15:29'!
paneColor
	"This is a much lighter variant of the base color used for interior panes"
	^self subclassResponsibility! !

!CWidgetLibrary methodsFor: 'lists' stamp: 'ar 5/24/2005 12:17'!
popUpListCostume
	^nil! !

!CWidgetLibrary methodsFor: 'basic' stamp: 'ar 1/22/2005 15:31'!
progressBarCostume
	^nil! !

!CWidgetLibrary methodsFor: 'buttons' stamp: 'ar 1/22/2005 15:32'!
pushButtonCostume
	^nil! !

!CWidgetLibrary methodsFor: 'buttons' stamp: 'ar 1/22/2005 15:32'!
radioButtonCostume
	^nil! !

!CWidgetLibrary methodsFor: 'scrolling' stamp: 'ar 1/22/2005 15:31'!
scrollBarCostume
	^nil! !

!CWidgetLibrary methodsFor: 'scrolling' stamp: 'bgf 6/23/2010 13:00'!
scrollBarHeight
	"Answer the default height for scrollbars"
	^16! !

!CWidgetLibrary methodsFor: 'scrolling' stamp: 'ar 1/22/2005 15:31'!
scrollPaneCostume
	^nil! !

!CWidgetLibrary methodsFor: 'colors' stamp: 'bgf 2/9/2009 01:09'!
scrollbarGradientColors
	"This describes the colors to use for scroll bar knobs"
	^{Color white. self baseColor}! !

!CWidgetLibrary methodsFor: 'colors' stamp: 'ar 1/22/2005 15:29'!
selectionColor
	"The base color for this scheme"
	^self subclassResponsibility! !

!CWidgetLibrary methodsFor: 'colors' stamp: 'bgf 2/9/2007 10:14'!
separatorColor
	"Answer the color for menu separators."
	self subclassResponsibility! !

!CWidgetLibrary methodsFor: 'basic' stamp: 'ar 1/22/2005 15:31'!
sliderCostume
	^nil! !

!CWidgetLibrary methodsFor: 'scrolling' stamp: 'ar 1/22/2005 15:31'!
spinnerCostume
	^nil! !

!CWidgetLibrary methodsFor: 'lists' stamp: 'ar 1/22/2005 15:31'!
spinnerListCostume
	^nil! !

!CWidgetLibrary methodsFor: 'lists' stamp: 'ar 1/22/2005 15:31'!
stringListCostume
	^nil! !

!CWidgetLibrary methodsFor: 'styling' stamp: 'ar 7/7/2005 09:21'!
styleSheetFor: aWidget
	"Answer the default style sheet applicable to aWidget"
	^nil! !

!CWidgetLibrary methodsFor: 'buttons' stamp: 'ar 1/22/2005 15:32'!
tabButtonCostume
	^nil! !

!CWidgetLibrary methodsFor: 'colors' stamp: 'ar 8/20/2007 12:16'!
textSelectionColor
	"Answer the color to be used for text selection"
	^self subclassResponsibility! !

!CWidgetLibrary methodsFor: 'lists' stamp: 'ar 1/22/2005 15:31'!
treeCostume
	^nil! !

!CWidgetLibrary methodsFor: 'windows' stamp: 'ar 1/22/2005 15:33'!
windowBorderStyle
	"window border style is style used for borders inside the window"
	^#simple! !

!CWidgetLibrary methodsFor: 'windows' stamp: 'ar 1/22/2005 15:33'!
windowBorderWidth
	"Answer the width to be used for windows and their elements"
	^1! !

!CWidgetLibrary methodsFor: 'colors' stamp: 'ar 6/2/2004 15:31'!
windowColor
	^self baseColor! !

!CWidgetLibrary methodsFor: 'windows' stamp: 'ar 1/22/2005 15:30'!
windowCostume
	^nil! !

!CWidgetLibrary methodsFor: 'windows' stamp: 'ar 1/22/2005 15:31'!
windowFrame
	^nil! !

!Behavior methodsFor: '*Tweak-Core-Proto' stamp: 'eem 5/13/2008 09:56'!
allFieldsReverseDo: aBlock
	"Evaluate aBlock with all field definitions, in superclass preceeding subclass order."

	"Nothing to do here; CProtoObject introduces field definitions."
	^self! !

!Behavior methodsFor: '*Tweak-Core-Proto' stamp: 'ar 5/7/2005 16:35'!
hiddenSubclasses
	"Uniclasses etc."
	^self obsoleteSubclasses! !

!Behavior methodsFor: '*Tweak-Core-Proto' stamp: 'ar 5/7/2005 16:34'!
methodTriggers
	^#()! !

WriteStream subclass: #CTextStream
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Core-Proto'!

!CTextStream methodsFor: 'as yet unclassified' stamp: 'ar 5/5/2004 17:59'!
applyAttribute: att beginningAt: startPos
	collection addAttribute: att from: startPos to: self position! !

!CTextStream methodsFor: 'as yet unclassified' stamp: 'ar 5/5/2004 19:12'!
growTo: anInteger
	" anInteger is the required minimal new size of the collection "
	| oldSize grownCollection newSize |
	oldSize := collection size.
	newSize := anInteger + (oldSize // 4 max: 20).
	grownCollection := collection class new: newSize.
	collection := grownCollection replaceFrom: 1 to: oldSize with: collection startingAt: 1.
	writeLimit := collection size.
! !

!CTextStream methodsFor: 'as yet unclassified' stamp: 'ar 5/5/2004 17:59'!
nextPutAll: aCollection 
	"Optimized access to get around Text at:Put: overhead"
	| n |
	n := aCollection size.
	position + n > writeLimit
		ifTrue:[self growTo: position + n + 10].
	collection 
		replaceFrom: position+1
		to: position + n
		with: aCollection
		startingAt: 1.
	position := position + n.! !

!CTextStream methodsFor: 'as yet unclassified' stamp: 'ar 5/5/2004 17:59'!
withAttribute: att do: strmBlock
	| pos1 val |
	pos1 := self position.
	val := strmBlock value.
	collection addAttribute: att from: pos1+1 to: self position.
	^ val! !

!CTextStream methodsFor: 'as yet unclassified' stamp: 'ar 5/5/2004 17:59'!
withAttributes: attributes do: streamBlock 
	| pos1 val |
	pos1 := self position.
	val := streamBlock value.
	attributes do: [:attribute |
		collection
			addAttribute: attribute
			from: pos1 + 1
			to: self position].
	^ val! !
CFieldDefinition initialize!
